/*
 * global -c と readline を用いた補完入力のためのサンプルプログラム: readtag
 */
#include <stdio.h>
#include <string.h>
#include <readline/readline.h>
/*
 * 候補生成関数
 *
 * text: 入力済みの文字列、state: 0 から始まる通し番号
 */
char *
gtags_generator(char *text, int state)
{
	static FILE *fp;
	char buf[1024];

	/* 初回呼び出し時の処理 */
	if (state == 0) {
		/* text で始まるシンボルの候補を生成する */
		snprintf(buf, sizeof(buf), "global -c %s", text);
		fp = popen(buf, "r");
		if (fp == NULL) {
			fprintf(stderr, "popen failed.\n");
			exit(1);
		}
	}
	/* 次の候補を一つ返す */
	if (fgets(buf, sizeof(buf), fp)) {
		buf[strlen(buf) - 1] = 0;  /* 改行を削除 */
		return strdup(buf);        /* malloc した領域にコピーして返す */
	}
	/* 候補がなくなったら NULL を返す */
	pclose(fp);
	return ((char *)NULL);
}
main (int argc, char **argv)
{
	char *line;
	/* 候補生成関数をreadlineに知らせる */
	rl_completion_entry_function = (rl_compentry_func_t *)gtags_generator;
	for (;;) {
		/* シンボルを一つ入力する */
		line = readline ("GTAGS: ");
		if (!line)
			break;
		/* 入力したシンボルを表示する */
		printf("%s\n", line);
		free(line);
	}
	exit(0);
}

